@@ -379,20 +379,17 @@ std::cerr << "article " << a.message_id << " references " << s << std::endl;
 void
 DataImpl :: load_part (const Quark          & group,
                        const Quark          & mid,
-                       size_t                 number,
-                       size_t                 lines,
-                       Article::Part        & new_part)
+                       int                    number,
+                       const StringView     & part_mid,
+                       unsigned long          lines,
+                       unsigned long          bytes)
 {
    GroupHeaders * h = get_group_headers (group);
    Article * a (h->find_article (mid));
+   pan_return_if_fail (a != 0);
 
-   pan_return_if_fail (a != NULL);
-
-   Article::Part& old_part (a->get_part (number));
-   if (old_part.empty()) {
-       old_part.swap (new_part);
-       a->lines += lines;
-   }
+   if (a->add_part (number, part_mid, bytes))
+     a->lines += lines;
 }
 
 namespace
@@ -472,6 +469,7 @@ DataImpl :: load_headers (const DataIO   & data_io,
       in->getline (line);
       //const unsigned long article_qty = view_to_ul (line); /* unused */
       const time_t now (time (0));
+      PartBatch part_batch;
       for (;;)
       {
         // look for the beginning of an Article record.
@@ -513,7 +511,7 @@ DataImpl :: load_headers (const DataIO   & data_io,
           if (tok.pop_token(server_tok,':') && tok.pop_token(group_tok,':')) {
             target_it->server = server_tok;
             target_it->group = group_tok.len==1 ? xref_lookup[(int)*group_tok.str] : Quark(group_tok);
-            target_it->number = view_to_ul (tok);
+            target_it->number = atoi (tok.str);
             const Server * server (find_server (target_it->server));
             if (server && ((!server->article_expiration_age) || (days_old <= server->article_expiration_age)))
               ++target_it;
@@ -534,11 +532,10 @@ DataImpl :: load_headers (const DataIO   & data_io,
           s.ltrim(); s.pop_token (tok); total_part_count = atoi(tok.str);
           s.ltrim(); s.pop_token (tok); found_part_count = atoi(tok.str);
         }
-        s.ltrim(); if (s.pop_token (tok)) a.lines = view_to_ul (tok); // this field was added in 0.115
-        if (!expired)
-          a.set_part_count (total_part_count);
+        s.ltrim(); if (s.pop_token (tok)) a.lines = atoi (tok.str); // this field was added in 0.115
 
         // found parts...
+        part_batch.init (a.message_id, total_part_count, found_part_count);
         for (int i(0), count(found_part_count); i<count; ++i)
         {
           const bool gotline (in->getline (s));
@@ -548,19 +545,25 @@ DataImpl :: load_headers (const DataIO   & data_io,
             StringView tok;
             s.ltrim ();
             s.pop_token (tok);
-            const unsigned long number (view_to_ul (tok));
-            if (!a.has_part(number)) { // corrupted entry
+            const int number (atoi (tok.str));
+            if (number > total_part_count) { // corrupted entry
               expired = true;
               break;
             }
-            Article::Part& p (a.get_part (number));
+            StringView part_mid;
+            unsigned long part_bytes (0);
             s.ltrim ();
-            s.pop_token (tok);
-            p.set_message_id (a.message_id, (tok.len==1 && *tok.str=='"') ? a.message_id.to_view() : tok);
-            s.pop_token(tok); p.bytes = view_to_ul (tok);
-            if (s.pop_token(tok)) a.lines += view_to_ul (tok); // this field was removed in 0.115
+            s.pop_token (part_mid);
+            if (part_mid.len==1 && *part_mid.str=='"')
+              part_mid = a.message_id.to_view ();
+            s.pop_token(tok); part_bytes = view_to_ul (tok);
+            part_batch.add_part (number, part_mid, part_bytes);
+
+            if (s.pop_token(tok)) a.lines += atoi (tok.str); // this field was removed in 0.115
           }
         }
+        if (!expired)
+          a.set_parts (part_batch);
 
         // add the article to the group if it hasn't all expired
         if (expired)
@@ -605,65 +608,59 @@ DataImpl :: load_headers (const DataIO   & data_io,
 
 namespace
 {
-  typedef Loki::AssocVector < pan::Quark, char > quark_to_symbol_t;
+  const char * lookup_symbols ("abcdefghijklmnopqrstuvwxyz"
+                               "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
+                               "1234567890!@#$%^&*()");
 
-  struct QuarkToSymbol {
-    ~QuarkToSymbol () {}
-    QuarkToSymbol (const quark_to_symbol_t& map): _map(map) { }
-    const quark_to_symbol_t _map;
-    const char* operator() (const Quark& quark) const {
+  struct QuarkToSymbol
+  {
+    typedef Loki::AssocVector < pan::Quark, char > quark_to_symbol_t;
+    quark_to_symbol_t _map;
+
+    const char* operator() (const Quark& quark) const
+    {
       static char buf[2];
       quark_to_symbol_t::const_iterator it (_map.find (quark));
       if (it == _map.end())
         return quark.c_str();
+
       buf[0] = it->second;
       buf[1] = '\0';
       return buf;
     }
+
+    void write (std::ostream& out, const StringView& comment) const
+    {
+      Quark quarks[UCHAR_MAX];
+      foreach_const (quark_to_symbol_t, _map, it)
+        quarks[(int)it->second] = it->first;
+
+      const size_t len (_map.size());
+      out << len;
+      if (!comment.empty())
+        out << "\t # " << comment << '\n';
+      for (size_t i(0); i!=len; ++i) {
+        const char ch (lookup_symbols[i]);
+        out << '\t' << ch << '\t' << quarks[(int)ch] << '\n';
+      }
+    }
   };
 
   typedef Loki::AssocVector<Quark,unsigned long> frequency_t;
 
-  const char * lookup_symbols ("abcdefghijklmnopqrstuvwxyz"
-                               "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
-                               "1234567890!@#$%^&*()");
-
-  QuarkToSymbol build_qts (frequency_t& freq)
+  void build_qts (frequency_t& freq, QuarkToSymbol& setme)
   {
-    quark_to_symbol_t qts_map;
-    for (int i(0), len(strlen(lookup_symbols)); i<len && !freq.empty(); ++i)
-    {
-      // find the most frequently-used entry still in freq
-      unsigned long max (0);
-      Quark q;
-      foreach_const (frequency_t, freq, it) {
-        if (it->second > max) {
-          max = it->second;
-          q = it->first;
-        }
-      }
-      //std::cerr << "[" << q << "] occurred " << max << " times" << std::endl;
-      qts_map[q] = lookup_symbols[i];
-      freq.erase (q);
-    }
+    setme._map.clear ();
+
+    typedef std::multimap<unsigned long,Quark> counts_t;
+    counts_t counts;
+    foreach_const (frequency_t, freq, it)
+      counts.insert (std::pair<unsigned long,Quark>(it->second,it->first));
+
+    counts_t::const_reverse_iterator it=counts.rbegin(), end=counts.rend();
+    for (const char * pch=lookup_symbols; *pch && it!=end; ++pch, ++it)
+      setme._map[it->second] = *pch;
     freq.clear ();
-    return QuarkToSymbol (qts_map);
-  }
-
-  void write_qts (std::ostream* out, const QuarkToSymbol& qts, const StringView& comment)
-  {
-    Quark quarks[UCHAR_MAX];
-    foreach_const (quark_to_symbol_t, qts._map, it)
-      quarks[(int)it->second] = it->first;
-
-    const size_t len (qts._map.size());
-    *out << len;
-    if (!comment.empty())
-      *out << "\t # " << comment << '\n';
-    for (size_t i(0); i!=len; ++i) {
-      const char ch (lookup_symbols[i]);
-      *out << '\t' << ch << '\t' << quarks[(int)ch] << '\n';
-    }
   }
 }
 
@@ -728,20 +725,21 @@ DataImpl :: save_headers (DataIO                       & data_io,
     foreach_const (std::vector<Article*>, articles, ait)
       foreach_const (Xref, (*ait)->xref, xit)
         ++frequency[xit->group];
-    const QuarkToSymbol xref_qts (build_qts (frequency));
-    write_qts (out, xref_qts, "xref shorthand count");
+    QuarkToSymbol xref_qts;
+    build_qts (frequency, xref_qts);
+    xref_qts.write (*out, "xref shorthand count");
 
     // author lookup section
     frequency.clear ();
     foreach_const (std::vector<Article*>, articles, ait)
       ++frequency[(*ait)->author];
-    const QuarkToSymbol author_qts (build_qts (frequency));
-    write_qts (out, author_qts, "author shorthand count");
+    QuarkToSymbol author_qts;
+    build_qts (frequency, author_qts);
+    author_qts.write (*out, "author shorthand count");
 
     // header section
     *out << articles.size() << endl;
     std::string references;
-    std::string part_mid;
     foreach_const (std::vector<Article*>, articles, ait)
     {
       ++article_count;
@@ -768,32 +766,20 @@ DataImpl :: save_headers (DataIO                       & data_io,
 
       // is_binary [total_part_count found_part_count]
       *out << (a->is_binary ? 't' : 'f');
-      if (a->is_binary) {
-        int foundPartCount (0);
-        foreach_const (Article::parts_t, a->parts, pit)
-          if (!pit->empty())
-            ++foundPartCount;
-        *out << ' ' << a->get_part_count() << ' ' << foundPartCount;
-      }
+      if (a->is_binary)
+        *out << ' ' << a->get_total_part_count() << ' ' << a->get_found_part_count();
       *out << ' ' << a->lines << '\n';
 
       // one line per foundPartCount (part-index message-id bytes lines)
-      int number (0);
-      bool first (true);
-      foreach_const (Article::parts_t, a->parts, pit) {
-        ++number;
-        if (!pit->empty()) {
-          ++part_count;
-          const Article::Part& p (*pit);
-          *out << '\t' << number << ' ';
-          p.get_message_id (message_id, part_mid);
-          if (message_id.to_view() == part_mid)
-            *out << '"';
-          else
-            *out << part_mid;
-          *out << ' ' << p.bytes << '\n';
-           first = false;
-        }
+      for (Article::part_iterator pit(a->pbegin()), end(a->pend()); pit!=end; ++pit) {
+        ++part_count;
+        const std::string& part_mid = pit.mid ();
+        *out << '\t' << pit.number() << ' ';
+        if (message_id.to_view() == part_mid)
+          *out << '"';
+        else
+          *out << part_mid;
+        *out << ' ' << pit.bytes() << '\n';
       }
     }
 
